[data-md-color-scheme="slate"] {
    --md-primary-fg-color: #ea4647;
    --md-primary-bg-color: #050811;
    --md-accent-fg-color: #fb923c;
    --md-default-bg-color: #050811;
    --md-code-bg-color: #0f1219;
}

[data-md-color-scheme="default"] {
    --md-primary-fg-color: #c83232;
    --md-accent-fg-color: #fb923c;
}
