"""정관에 관한 사항 분석 모듈."""

from dartlab.engines.docsParser.finance.articlesOfIncorporation.pipeline import articlesOfIncorporation
from dartlab.engines.docsParser.finance.articlesOfIncorporation.types import ArticlesResult

__all__ = ["articlesOfIncorporation", "ArticlesResult"]
