from dartlab.engines.docsParser.finance.executive.pipeline import executive
from dartlab.engines.docsParser.finance.executive.types import ExecutiveResult

__all__ = [
    "executive",
    "ExecutiveResult",
]
