"""기타 재무에 관한 사항 분석 모듈."""

from dartlab.engines.docsParser.finance.otherFinance.pipeline import otherFinance
from dartlab.engines.docsParser.finance.otherFinance.types import OtherFinanceResult

__all__ = ["otherFinance", "OtherFinanceResult"]
