"""감사제도에 관한 사항 분석 모듈."""

from dartlab.engines.docsParser.finance.auditSystem.pipeline import auditSystem
from dartlab.engines.docsParser.finance.auditSystem.types import AuditSystemResult

__all__ = ["auditSystem", "AuditSystemResult"]
