from dartlab.engines.docsParser.finance.dividend.pipeline import dividend
from dartlab.engines.docsParser.finance.dividend.types import DividendResult

__all__ = [
    "dividend",
    "DividendResult",
]
