from dartlab.engines.docsParser.finance.employee.pipeline import employee
from dartlab.engines.docsParser.finance.employee.types import EmployeeResult

__all__ = [
    "employee",
    "EmployeeResult",
]
